Changed human_num() to big_num() with an extra arg so that it can
[rsync/rsync.git] / log.c
diff --git a/log.c b/log.c
index b837900..2d828c1 100644 (file)
--- a/log.c
+++ b/log.c
@@ -3,11 +3,12 @@
  *
  * Copyright (C) 1998-2001 Andrew Tridgell <tridge@samba.org>
  * Copyright (C) 2000-2001 Martin Pool <mbp@samba.org>
- * Copyright (C) 2003-2007 Wayne Davison
+ * Copyright (C) 2003-2008 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ * with this program; if not, visit the http://fsf.org website.
  */
 
 #include "rsync.h"
+#include "ifuncs.h"
 
-extern int verbose;
 extern int dry_run;
 extern int am_daemon;
 extern int am_server;
@@ -34,12 +34,13 @@ extern int msg_fd_out;
 extern int allow_8bit_chars;
 extern int protocol_version;
 extern int preserve_times;
-extern int preserve_uid;
-extern int preserve_gid;
+extern int uid_ndx;
+extern int gid_ndx;
 extern int stdout_format_has_i;
 extern int stdout_format_has_o_or_i;
 extern int logfile_format_has_i;
 extern int logfile_format_has_o_or_i;
+extern int receiver_symlink_times;
 extern mode_t orig_umask;
 extern char *auth_user;
 extern char *stdout_format;
@@ -52,6 +53,7 @@ extern iconv_t ic_chck;
 extern iconv_t ic_send, ic_recv;
 #endif
 extern char curr_dir[];
+extern char *module_dir;
 extern unsigned int module_dirlen;
 
 static int log_initialised;
@@ -59,7 +61,8 @@ static int logfile_was_closed;
 static FILE *logfile_fp;
 struct stats stats;
 
-int log_got_error = 0;
+int got_xfer_error = 0;
+int output_needs_newline = 0;
 
 struct {
         int code;
@@ -84,6 +87,7 @@ struct {
        { RERR_PARTIAL    , "some files could not be transferred" },
        { RERR_VANISHED   , "some files vanished before they could be transferred" },
        { RERR_TIMEOUT    , "timeout in data send/receive" },
+       { RERR_CONTIMEOUT , "timeout waiting for daemon connection" },
        { RERR_CMD_FAILED , "remote shell failed" },
        { RERR_CMD_KILLED , "remote shell killed" },
        { RERR_CMD_RUN    , "remote command could not be run" },
@@ -233,8 +237,8 @@ static void filtered_fwrite(FILE *f, const char *buf, int len, int use_isprint)
 }
 
 /* this is the underlying (unformatted) rsync debugging function. Call
* it with FINFO, FERROR or FLOG.  Note: recursion can happen with
* certain fatal conditions. */
 void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
 {
        int trailing_CR_or_NL;
@@ -257,7 +261,7 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
                return;
        }
 
-       if (code == FSOCKERR) /* This gets simplified for a non-sibling. */
+       if (code == FERROR_SOCKET) /* This gets simplified for a non-sibling. */
                code = FERROR;
 
        if (code == FCLIENT)
@@ -265,7 +269,7 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
        else if (am_daemon || logfile_name) {
                static int in_block;
                char msg[2048];
-               int priority = code == FERROR ? LOG_WARNING : LOG_INFO;
+               int priority = code == FINFO || code == FLOG ? LOG_INFO :  LOG_WARNING;
 
                if (in_block)
                        return;
@@ -281,12 +285,19 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
        } else if (code == FLOG)
                return;
 
-       if (quiet && code != FERROR)
+       if (quiet && code == FINFO)
                return;
 
        if (am_server) {
+               enum msgcode msg = (enum msgcode)code;
+               if (protocol_version < 30) {
+                       if (msg == MSG_ERROR)
+                               msg = MSG_ERROR_XFER;
+                       else if (msg == MSG_WARNING)
+                               msg = MSG_INFO;
+               }
                /* Pass the message to the non-server side. */
-               if (send_msg((enum msgcode)code, buf, len, !is_utf8))
+               if (send_msg(msg, buf, len, !is_utf8))
                        return;
                if (am_daemon) {
                        /* TODO: can we send the error to the user somehow? */
@@ -295,8 +306,11 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
        }
 
        switch (code) {
+       case FERROR_XFER:
+               got_xfer_error = 1;
+               /* FALL THROUGH */
        case FERROR:
-               log_got_error = 1;
+       case FWARNING:
                f = stderr;
                break;
        case FINFO:
@@ -306,31 +320,41 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
                exit_cleanup(RERR_MESSAGEIO);
        }
 
+       if (output_needs_newline) {
+               fputc('\n', f);
+               output_needs_newline = 0;
+       }
+
        trailing_CR_or_NL = len && (buf[len-1] == '\n' || buf[len-1] == '\r')
                          ? buf[--len] : 0;
 
+       if (len && buf[0] == '\r') {
+               fputc('\r', f);
+               buf++;
+               len--;
+       }
+
 #ifdef ICONV_CONST
        if (ic != (iconv_t)-1) {
+               xbuf outbuf, inbuf;
                char convbuf[1024];
-               ICONV_CONST char *in_buf = (ICONV_CONST char *)buf;
-               char *out_buf = convbuf;
-               size_t in_cnt = len, out_cnt = sizeof convbuf - 1;
-
-               iconv(ic, NULL, 0, NULL, 0);
-               while (iconv(ic, &in_buf,&in_cnt,
-                            &out_buf,&out_cnt) == (size_t)-1) {
-                       if (out_buf != convbuf) {
-                               filtered_fwrite(f, convbuf, out_buf - convbuf, 0);
-                               out_buf = convbuf;
-                               out_cnt = sizeof convbuf - 1;
+               int ierrno;
+
+               INIT_CONST_XBUF(outbuf, convbuf);
+               INIT_XBUF(inbuf, (char*)buf, len, -1);
+
+               while (inbuf.len) {
+                       iconvbufs(ic, &inbuf, &outbuf, 0);
+                       ierrno = errno;
+                       if (outbuf.len) {
+                               filtered_fwrite(f, convbuf, outbuf.len, 0);
+                               outbuf.len = 0;
                        }
-                       if (errno == E2BIG)
+                       if (!ierrno || ierrno == E2BIG)
                                continue;
-                       fprintf(f, "\\#%03o", *(uchar*)in_buf++);
-                       in_cnt--;
+                       fprintf(f, "\\#%03o", CVAL(inbuf.buf, inbuf.pos++));
+                       inbuf.len--;
                }
-               if (out_buf != convbuf)
-                       filtered_fwrite(f, convbuf, out_buf - convbuf, 0);
        } else
 #endif
                filtered_fwrite(f, buf, len, !allow_8bit_chars);
@@ -341,8 +365,8 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
        }
 }
 
/* This is the rsync debugging function. Call it with FINFO, FERROR or
* FLOG. */
 void rprintf(enum logcode code, const char *format, ...)
 {
        va_list ap;
@@ -420,10 +444,10 @@ void rflush(enum logcode code)
        if (am_daemon || code == FLOG)
                return;
 
-       if (code == FERROR || am_server)
-               f = stderr;
-       else
+       if (!am_server && (code == FINFO || code == FCLIENT))
                f = stdout;
+       else
+               f = stderr;
 
        fflush(f);
 }
@@ -483,11 +507,11 @@ static void log_formatted(enum logcode code, const char *format, const char *op,
                case 'U':
                        strlcat(fmt, "u", sizeof fmt);
                        snprintf(buf2, sizeof buf2, fmt,
-                                preserve_uid ? F_OWNER(file) : 0);
+                                uid_ndx ? F_OWNER(file) : 0);
                        n = buf2;
                        break;
                case 'G':
-                       if (!preserve_gid || file->flags & FLAG_SKIP_GROUP)
+                       if (!gid_ndx || file->flags & FLAG_SKIP_GROUP)
                                n = "DEFAULT";
                        else {
                                strlcat(fmt, "u", sizeof fmt);
@@ -504,7 +528,7 @@ static void log_formatted(enum logcode code, const char *format, const char *op,
                        break;
                case 'M':
                        n = c = timestring(file->modtime);
-                       while ((c = strchr(p, ' ')) != NULL)
+                       while ((c = strchr(c, ' ')) != NULL)
                                *c = '-';
                        break;
                case 'B':
@@ -580,7 +604,7 @@ static void log_formatted(enum logcode code, const char *format, const char *op,
                        n = timestring(time(NULL));
                        break;
                case 'P':
-                       n = lp_path(module_id);
+                       n = module_dir;
                        break;
                case 'u':
                        n = auth_user;
@@ -611,7 +635,7 @@ static void log_formatted(enum logcode code, const char *format, const char *op,
                        break;
                case 'i':
                        if (iflags & ITEM_DELETED) {
-                               n = "*deleting";
+                               n = "*deleting  ";
                                break;
                        }
                        n  = c = buf2 + MAXPATHLEN - 32;
@@ -619,14 +643,21 @@ static void log_formatted(enum logcode code, const char *format, const char *op,
                              ? iflags & ITEM_XNAME_FOLLOWS ? 'h' : 'c'
                             : !(iflags & ITEM_TRANSFER) ? '.'
                             : !local_server && *op == 's' ? '<' : '>';
-                       c[1] = S_ISDIR(file->mode) ? 'd'
-                            : IS_SPECIAL(file->mode) ? 'S'
-                            : IS_DEVICE(file->mode) ? 'D'
-                            : S_ISLNK(file->mode) ? 'L' : 'f';
-                       c[2] = !(iflags & ITEM_REPORT_CHECKSUM) ? '.' : 'c';
-                       c[3] = !(iflags & ITEM_REPORT_SIZE) ? '.' : 's';
-                       c[4] = !(iflags & ITEM_REPORT_TIME) ? '.'
-                            : !preserve_times || S_ISLNK(file->mode) ? 'T' : 't';
+                       if (S_ISLNK(file->mode)) {
+                               c[1] = 'L';
+                               c[3] = '.';
+                               c[4] = !(iflags & ITEM_REPORT_TIME) ? '.'
+                                    : !preserve_times || !receiver_symlink_times
+                                   || (iflags & ITEM_REPORT_TIMEFAIL) ? 'T' : 't';
+                       } else {
+                               c[1] = S_ISDIR(file->mode) ? 'd'
+                                    : IS_SPECIAL(file->mode) ? 'S'
+                                    : IS_DEVICE(file->mode) ? 'D' : 'f';
+                               c[3] = !(iflags & ITEM_REPORT_SIZE) ? '.' : 's';
+                               c[4] = !(iflags & ITEM_REPORT_TIME) ? '.'
+                                    : !preserve_times ? 'T' : 't';
+                       }
+                       c[2] = !(iflags & ITEM_REPORT_CHANGE) ? '.' : 'c';
                        c[5] = !(iflags & ITEM_REPORT_PERMS) ? '.' : 'p';
                        c[6] = !(iflags & ITEM_REPORT_OWNER) ? '.' : 'o';
                        c[7] = !(iflags & ITEM_REPORT_GROUP) ? '.' : 'g';
@@ -735,7 +766,7 @@ void maybe_log_item(struct file_struct *file, int iflags, int itemizing,
 {
        int significant_flags = iflags & SIGNIFICANT_ITEM_FLAGS;
        int see_item = itemizing && (significant_flags || *buf
-               || stdout_format_has_i > 1 || (verbose > 1 && stdout_format_has_i));
+               || stdout_format_has_i > 1 || (INFO_GTE(NAME, 2) && stdout_format_has_i));
        int local_change = iflags & ITEM_LOCAL_CHANGE && significant_flags;
        if (am_server) {
                if (logfile_name && !dry_run && see_item
@@ -759,7 +790,7 @@ void log_delete(const char *fname, int mode)
 
        x.file.mode = mode;
 
-       if (!verbose && !stdout_format)
+       if (!INFO_GTE(DEL, 1) && !stdout_format)
                ;
        else if (am_server && protocol_version >= 29 && len < MAXPATHLEN) {
                if (S_ISDIR(mode))
@@ -787,11 +818,11 @@ void log_delete(const char *fname, int mode)
 void log_exit(int code, const char *file, int line)
 {
        if (code == 0) {
-               rprintf(FLOG,"sent %.0f bytes  received %.0f bytes  total size %.0f\n",
-                       (double)stats.total_written,
-                       (double)stats.total_read,
-                       (double)stats.total_size);
-       } else {
+               rprintf(FLOG,"sent %s bytes  received %s bytes  total size %s\n",
+                       big_num(stats.total_written, 0),
+                       big_num(stats.total_read, 0),
+                       big_num(stats.total_size, 0));
+       } else if (am_server != 2) {
                const char *name;
 
                name = rerr_name(code);
@@ -800,7 +831,7 @@ void log_exit(int code, const char *file, int line)
 
                /* VANISHED is not an error, only a warning */
                if (code == RERR_VANISHED) {
-                       rprintf(FINFO, "rsync warning: %s (code %d) at %s(%d) [%s=%s]\n",
+                       rprintf(FWARNING, "rsync warning: %s (code %d) at %s(%d) [%s=%s]\n",
                                name, code, file, line, who_am_i(), RSYNC_VERSION);
                } else {
                        rprintf(FERROR, "rsync error: %s (code %d) at %s(%d) [%s=%s]\n",
@@ -233,8 +237,8 @@ static void filtered_fwrite(FILE *f, const char *buf, int len, int use_isprint)
 
 /* this is the underlying (unformatted) rsync debugging function. Call
- * it with FINFO, FERROR or FLOG.  Note: recursion can happen with
- * certain fatal conditions. */
+ * it with FINFO, FERROR_*, FWARNING, FLOG, or FCLIENT.  Note: recursion
+ * can happen with certain fatal conditions. */
 void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
 {
@@ -341,8 +365,8 @@ void rwrite(enum logcode code, const char *buf, int len, int is_utf8)
 }
 
-/* This is the rsync debugging function. Call it with FINFO, FERROR or
- * FLOG. */
+/* This is the rsync debugging function. Call it with FINFO, FERROR_*,
+ * FWARNING, FLOG, or FCLIENT. */
 void rprintf(enum logcode code, const char *format, ...)
 {